
#include "common.hxx"
#include "asteroid.hxx"

asteroid::asteroid()		// Constructor
{
	scaleX = 1.0;		// Normal scale is 1.0 
	scaleY = 1.0;
}

// read the input file containing asteroid coordinates.	
int asteroid::readInput(const char* filename)
{
	ctr = 0;		// ctr stores the number of vertices
	int ret = TRUE;
	int n,tmp_x,tmp_y;	// tmp variables
	FILE* fp = fopen(filename,"r");		// read the asteroids vertices from file
	if (!fp) {
		printf("Couldn't open file\n");
		ret = FALSE;
	}
	else {
		for (int i=0;!feof(fp);i=i+2)
		{
			n=fscanf(fp,"%f %f\n", &this->vertices[i], &this->vertices[i+1]);	// store them in vertices array
			ctr=ctr+2;								// i element has x coordinate
		}										// i+ 1 has y coordinate
		ret = TRUE;
	}
	return(ret);	
}

// draw the asteroid.
void asteroid::draw()
{
	glPushMatrix();			// do transformations
		glColor3f(R,G,B);	// color
		glTranslatef(this->deltaX, this->deltaY, 0);	// translate
		glScalef(scaleX,scaleY,0);			// scale
		glRotatef(this->angle,0.0,0.0,1.0);		// rotate
		glBegin(GL_LINE_LOOP);				// use lines
			for (int i = 0;i < this->ctr;i=i+2)
				glVertex2f(this->vertices[i], this->vertices[i+1]);
		glEnd();
	glPopMatrix();
}	

// draws an explosion.
void asteroid::drawExplosion()
{
	int rad = 0;
	glPushMatrix();
		glColor3f(1.0,1.0,0.0);	// draw at the same location as the asteroid
		glTranslatef(this->deltaX, this->deltaY, 0);
		glScalef(3.0,3.0,0);
		glRotatef(this->angle,0.0,0.0,1.0);
		glBegin(GL_LINE_LOOP);
			for (int i = 0;i < 3;i++)	// draw concentric circles
			{
				rad+=2;				
				for (float theta = 0;theta < (2*PI);theta+= (PI/180))
					glVertex2f(rad *cosf(theta), rad *sinf(theta));
			}	
		glEnd();
	glPopMatrix();
}	

// color the asteroid.
void asteroid::setColor(float red, float green, float blue)
{
	R = red;
	G = green;
	B = blue;
}

// sets the scaling factor of the asteroid.
void asteroid::scale(float value)
{
	scaleX = scaleY = value;
}

// returns the direction in which asteroid turns (-1 or 1).
int asteroid::getTurnDir()
{
	return turnDir;
}

// set the direction in which the asteroid will turn.
void asteroid::setTurnDir(int value)
{
	turnDir = value;
}

// returns the angle of rotation of the asteroid.
float asteroid::getAngle()
{
	return angle;
}

// returns the x coordinate of the centre of the asteroid.
float asteroid::getdeltaX()
{
	return deltaX;
}	

//  returns the y coordinate of the centre of the asteroid.
float asteroid::getdeltaY()
{
	return deltaY;
}	

// initialises the asteroid centre to origin.
void asteroid::resetDeltaXY()
{
	deltaX = 0;
	deltaY = 0;
}

// returns the scale factor in the x direction.
float asteroid::getScaleX()
{
	return scaleX;
}

// returns the scale factor in the y direction.
float asteroid::getScaleY()
{
	return scaleY;
}

// set deltaX.
void asteroid::SetDeltaX(float X)
{
	deltaX = X;
}

// set deltaY.	
void asteroid::SetDeltaY(float Y)
{
	deltaY = Y;
}
	
// perform some initializations.
void asteroid::setFirstTime()
{
	goRight = TRUE;		// set some flags so that asteroids know where
	goLeft = FALSE;		// to go
	goDown = TRUE;
	goUp = FALSE;	
}

// returns the radius of the asteroid.
float asteroid::getRadius()
{
	return radius;
}

// sets the hitByMissile flag to indicate that asteroid has been shot.
void asteroid::setHitByMissile()
{
	hit_by_missile = TRUE;
}

// resets the hitByMissile flag.
void asteroid::resetHitByMissile()
{
	hit_by_missile = FALSE;
}

// returns whether asteroid was hit or not.	
int asteroid::getHitByMissile()
{
	return hit_by_missile;
}

// set the killed flag to indicate that asteroid is destroyed.
void asteroid::setKilled()	
{
	killed = TRUE;
}

// reset the killed flag to indicate that asteroid is now active again.
void asteroid::resetKilled()	
{
	killed = FALSE;
}

// returns the status of the asteroid - whether it is killed or not.
int asteroid::getKilled()
{
	return killed;
}

// move the asteroid in the x direction.	
void asteroid::moveX(float X)
{
	// check collisions with the right edge
	if (((deltaX + radius) > RIGHT_EDGE - X) && (goRight == TRUE))
	{
		goRight = FALSE;
		goLeft = TRUE;
	}
	// check collisions with the left edge
	if (((deltaX - radius) < LEFT_EDGE + X) && (goLeft == TRUE))
	{
		goRight = TRUE;
		goLeft = FALSE;
	}
	if (goRight)
		deltaX = deltaX + X;

	if (goLeft)
		deltaX = deltaX - X;


}

// move the asteroid in the y direction.		
void asteroid::moveY(float Y)
{
	// check collision with bottom edge
	if (((deltaY + radius) > BOTTOM_EDGE - Y) && (goDown == TRUE))
	{
		goDown = FALSE;
		goUp = TRUE;
	}
	
	// check collision with top edge
	if (((deltaY - radius) < TOP_EDGE + Y) && (goUp == TRUE))
	{
		goDown = TRUE;
		goUp = FALSE;
	}
	
	if (goDown)
		deltaY = deltaY + Y;

	if (goUp)
		deltaY = deltaY - Y;

}

// rotate the asteroid in the direction specified by the sign on delta.
void asteroid::Turn(float delta)
{
	turnDir = (int) delta;
	angle = angle + delta;
}			
	
// calculate the radius of the asteroid.
float asteroid::CalculateRadius() 
{
	// here we find the radius by averaging out the distances from the
	// centre to each vertex.
	float distance = 0;
	for (int i = 0;i < this->ctr; i+=2)
		distance = distance + sqrt( ((this->vertices[i]) * (this->vertices[i])) + ((this->vertices[i+1]) * (this->vertices[i+1])));
	return radius = distance/(ctr/2);
}

	

		
